#include "stdafx.hpp"

#include "XMLWriter.hpp"

#include <vector>

namespace
{
	struct AttributeHolder
	{
		AttributeHolder()
		{
		}

		AttributeHolder( const AttributeHolder& v_other )
			: ns( v_other.ns )
			, localName( v_other.localName )
			, qName( v_other.qName )
			, type( v_other.type )
			, value( v_other.value )
		{
		}

		AttributeHolder& operator=( const AttributeHolder& v_other ) {
			if( this != &v_other ) {
				ns = v_other.ns;
				localName = v_other.localName;
				qName = v_other.qName;
				type = v_other.type;
				value = v_other.value;
			}
			return *this;
		}

		tstring ns;
		tstring localName;
		tstring qName;
		tstring type;
		tstring value;
	};

	typedef std::vector<AttributeHolder> AttributeHolderList;

	class DefaultSAXAttributesHolder : public SAXAttributesHolder
	{
	public:
		
		DefaultSAXAttributesHolder()
		{
		}
		
		DefaultSAXAttributesHolder( const DefaultSAXAttributesHolder& v_other )
			: attributeHolderList_( v_other.attributeHolderList_ )
		{
		}

		DefaultSAXAttributesHolder& operator=( const DefaultSAXAttributesHolder& v_other )
		{
			if( this != &v_other ) {
				attributeHolderList_ = v_other.attributeHolderList_;
			}
			return * this;
		}

		virtual ~DefaultSAXAttributesHolder()
		{
		}

		virtual void clear()
		{
			attributeHolderList_.clear();
		}

		virtual void addAttribute(
			const tstring& v_ns,
			const tstring& v_localName,
			const tstring& v_qName,
			const tstring& v_type,
			const tstring& v_value )
		{
			AttributeHolder attr;
			attr.ns = v_ns;
			attr.localName = v_localName;
			attr.qName = v_qName;
			attr.type = v_type;
			attr.value = v_value;
			attributeHolderList_.push_back( attr );
		}

		virtual void removeAttribute( int v_index )
		{
			attributeHolderList_.erase( attributeHolderList_.begin() + v_index );
		}

		virtual void setAttribute(
			int v_index,
			const tstring& v_ns,
			const tstring& v_localName,
			const tstring& v_qName,
			const tstring& v_type,
			const tstring& v_value )
		{
			AttributeHolder attr;
			attr.ns = v_ns;
			attr.localName = v_localName;
			attr.qName = v_qName;
			attr.type = v_type;
			attr.value = v_value;
			attributeHolderList_[ v_index ] = attr;
		}

		///

		virtual int getLength() const
		{
			return 0;
		}

		virtual tstring getURI(int) const
		{
			return _TEXT("");
		}

		virtual tstring getLocalName(int) const
		{
			return _TEXT("");
		}

		virtual tstring getQName(int) const
		{
			return _TEXT("");
		}

		virtual void getName(int, tstring*, tstring*, tstring* ) const
		{
		}

		virtual int getIndexFromName( const tstring&, const tstring& ) const
		{
			return 0;
		}

		virtual int getIndexFromQName( const tstring& ) const
		{
			return 0;
		}

		virtual tstring getType(int) const
		{
			return _TEXT("");
		}

		virtual tstring getTypeFromName( const tstring&, const tstring& ) const
		{
			return _TEXT("");
		}

		virtual tstring getTypeFromQName( const tstring& ) const 
		{
			return _TEXT("");
		}

		virtual tstring getValue(int) const
		{
			return _TEXT("");
		}

		virtual tstring getValueFromName( const tstring&, const tstring& ) const
		{
			return _TEXT("");
		}

		virtual tstring getValueFromQName( const tstring& ) const
		{
			return _TEXT("");
		}

	private:
		AttributeHolderList attributeHolderList_;
	};

	class DefaultXMLWriter : public XMLWriter
	{
	public:
		virtual SAXAttributesHolderPtr createAttributesHolder()
		{
			return SAXAttributesHolderPtr( new DefaultSAXAttributesHolder() );
		}

		virtual void startDocument()
		{
		}

		virtual void endDocument()
		{
		}

		virtual void startPrefixMapping(const tstring&, const tstring& )
		{
		}

		virtual void endPrefixMapping(const tstring& )
		{
		}

		virtual void startElement(
			const tstring& v_namespace,
			const tstring& v_localName,
			const tstring& v_QName,
			const SAXAttributes& pAttr
			)
		{
		}

		virtual void endElement(
			const tstring& v_namespace,
			const tstring& v_localName,
			const tstring& v_QName
			)
		{
		}

		virtual void characters(const tstring& v_chars )
		{
		}

		virtual void ignorableWhitespace(const tstring& v_chars )
		{
		}

		virtual void processingInstruction( const tstring& , const tstring& )
		{
		}

		virtual void skippedEntity( const tstring& )
		{
		}

	};
}


XMLWriterFactory::XMLWriterFactory()
{
}

XMLWriterFactory::~XMLWriterFactory()
{
}

XMLWriterPtr XMLWriterFactory::create( const tstring& v_fileName )
{
	return XMLWriterPtr( new DefaultXMLWriter() );
}

